import os
import itertools
import uuid
import unidecode
from datetime import datetime,date
from django.db import models
from django.contrib.auth import get_user_model
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.utils.text import slugify
from django.core.mail import EmailMessage
from django.utils.translation import ugettext as _


from django import forms
from djangoldp.models import Model
from djangoldp_conversation.models import Conversation

from djangoldp_i18n.views import I18nLDPViewSet
from django_countries.fields import CountryField,countries
from django_countries.widgets import CountrySelectWidget



from django.conf import settings

User = get_user_model()


def cleanAccentuation(text):
    text = unidecode.unidecode(text)
    return text


# appending custom behaviour to User model
def active_actions(self):
    return self.actiongroups.filter(actiondate__gte=date.today())

User.active_actions=active_actions

def convergent_actions(self):
    start_date = datetime(2019, 11, 28)
    end_date = datetime(2019, 12, 9)
    return self.actiongroups.filter(actiondate__gt=start_date,actiondate__lt=end_date)

User.convergent_actions=convergent_actions


class Countrie(Model):
    name = models.CharField(max_length=255, verbose_name=u"Country Code")
    country = CountryField(blank_label='(select country)',countries_flag_url='/static/flags/img/{code}.svg')
    flag = models.CharField(max_length=250,verbose_name=u"Flag", blank=True, null=True)
    flag_css = models.CharField(max_length=250,verbose_name=u"Flag css", blank=True, null=True)

    class Meta: 
        serializer_fields = ['name','country','flag','flag_css']
        nested_fields= ['action_country','representative_country']
    def __str__(self):
        return self.country.code

@receiver(post_save, sender=Countrie)
def save_country_flag(sender, instance,created, **kwargs):
    #print("old names",instance.name,instance.country,instance.country.code)

    if  created:
        if instance.name == 'Global':
            instance.flag = '/static/flags/img/global.svg'
            instance.save()
        else:
            instance.name = instance.country.code
            instance.flag = instance.country.flag
            instance.flag_css = instance.country.flag_css
            #print("new names",instance.name,instance.country,instance.country.code)
            instance.save()


class Tag(Model):
  name = models.CharField(max_length=250, verbose_name=u"Tag name")
  class Meta:
    serializer_fields = ['@id','name']
  def __str__(self):
      return self.name

class visualArguementToDownload(Model):
  name = models.CharField(max_length=150, verbose_name=u"titre de l'argument")
  content = models.TextField(verbose_name=u"Description de l'argument")
  image = models.ImageField(max_length=150, verbose_name=u"Image")
  country = models.ManyToManyField(Countrie, related_name="reasonImage", verbose_name=u"Langue")
  source = models.URLField(verbose_name=u"source", null=True, blank=True)
  tag = models.ManyToManyField(Tag,verbose_name=u"Type d'argument")

  class Meta:
    serializer_fields = ['@id','name','content','source','tag','image','country']
    anonymous_perms = ['view']
    authenticated_perms = ['inherit', 'add']
    ordering = ['?']

  def __str__(self):
    return self.name

class commissionPermanente(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = ['commissionPermanente']
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name

class missionParlementaire(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = ['missionParlementaire']
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']
    
    def __str__(self):
        return self.name



class additionalFunction(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = []
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name


class groupeEtude(Model):
    name = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom de la commission")
    
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = []
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name


class parliamentaryPosition(Model):
    name = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Nom de la commission")
    class Meta:
        serializer_fields = ['@id', 'name']
        nested_fields = []
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name

class representativeFunction(Model):
    name = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Fonction de l'élu")

    class Meta:
        nested_fields = []
        authenticated_perms = ['inherit', 'add', 'change']

    def __str__(self):
        return self.name

class Representative(Model):
    """Displaying contact and functions of elected officials"""
    #global information
    civility = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Civilité")
    firstname = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Prénom")
    lastname = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Nom")
    fullname = models.CharField(max_length=500, blank=True, null=True, verbose_name=u"Nom complet")
    function = models.ForeignKey(representativeFunction,blank=True,null=True, related_name="representativePosition", verbose_name=u"Fonction",
                                       on_delete=models.CASCADE)
    title = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Titre")
    politicalParty = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Parti politique")
    photo = models.ImageField(verbose_name=u"Photo")
    slug = models.SlugField(blank=True, null=True, unique=True)
    
    #Location info
    election_department = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Département d'élection")
    election_region = models.CharField(max_length=250, blank=True, null=True, verbose_name=u"Région d'élection")
    address1 = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse 1")
    address2 = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse 2")
    postal_code = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Code postal")
    city = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Ville")
    allPostalCodes = models.TextField(blank=True, verbose_name=u"Tous les codes postaux de la circonscription")
    #link to country
    representativeCountry = models.ForeignKey(Countrie, related_name='representative_country', blank=True, null=True, verbose_name=u"Pays",on_delete=models.SET_NULL)
    
    #Activity information
    permanentCommission = models.ForeignKey(commissionPermanente,blank=True,null=True, related_name="commissionPermanente", verbose_name=u"Commission permanente",
                                       on_delete=models.CASCADE)
    permanentCommissionFunction = models.ForeignKey(parliamentaryPosition,blank=True,null=True, related_name="commissionPermanenteFunction", verbose_name=u"Fonction commission permanente",
                                       on_delete=models.CASCADE)

    parliamentaryMissions = models.ManyToManyField(missionParlementaire,blank=True, related_name="missionParlementaire", verbose_name=u"missions parlementaires")
    
    additionalFunctions = models.ManyToManyField(additionalFunction,blank=True, related_name="additionalFunction", verbose_name=u"Fonctions additionnels") 
    
    studyGroups = models.ManyToManyField(groupeEtude,blank=True, related_name="study_groups", verbose_name=u"Groupes d'études")
    
    #Contact information
    assistants = models.ManyToManyField('Representative', blank=True)

    #position on campaign -> needs to be refactored to allow this info
    #to be changed depending on the action
    position_campagne = models.CharField(max_length=255, blank=True, verbose_name=u"Position de l'élue sur la campange")

        
    #new fields
    circonscription = models.CharField(max_length=50, blank=True, null=True, verbose_name=u"Circonscription")
    profession = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Metier")
    facebook = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Facebook")
    website = models.CharField(max_length=300, blank=True, null=True, verbose_name=u"Site web")

    class Meta:
        serializer_fields = ['@id', 'civility','facebook','profession','circonscription', 'firstname', 'lastname', 'fullname', 'function', 'title', 'politicalParty', 'election_department', 'election_region', 'directPhones','directEmails','twitterAccount', 'address1', 'address2', 'postal_code', 'city', 'representativeCountry', 'website', 'photo', 'slug', 'permanentCommission','permanentCommissionFunction', 'additionalFunctions', 'parliamentaryMissions', 'studyGroups', 'assistants','allPostalCodes','position_campagne']
        nested_fields = ["directPhones","directEmails","twitterAccount"]
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add', 'change']
        ordering = ['fullname']
    """
    def fullname(self):
        self.fullname = "{} {}".format(self.firstname, self.lastname)
        return "{} {}".format(self.firstname, self.lastname)
    """
    def __str__(self):
        if self.firstname and self.lastname:
            return self.firstname + " " + self.lastname
        elif self.fullname:
            return self.fullname
        else:
            return "no name"

class Phone(Model):
    """Phone numbers of elected officials"""
    name = models.CharField(max_length=50, blank=True, null=True, verbose_name=u"Numéro de téléphone")
    representative = models.ForeignKey(Representative,blank=True, null=True, related_name="directPhones", verbose_name=u"Élu",
                                       on_delete=models.CASCADE)
    def __str__(self):
        return self.name

class Email(Model):
    """Emails of elected officials"""
    name = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Email")
    representative = models.ForeignKey(Representative,blank=True, null=True, related_name="directEmails", verbose_name=u"Elu",
                                       on_delete=models.CASCADE)
    class Meta(Model.Meta):
        container_path = '/api/directEmails/'

    def __str__(self):
        return self.name

class TwitterAccount(Model):
    """Emails of elected officials"""
    name = models.CharField(max_length=150, blank=True, null=True, verbose_name=u"Compte twitter")
    representative = models.ForeignKey(Representative,blank=True, null=True, related_name="twitterAccount", verbose_name=u"Elu",
                                       on_delete=models.CASCADE)

    def __str__(self):
        return self.name




# TODO: leverage Loomio API to add loomio topics within an action group
class LoomioTopic(Model):
    title = models.CharField(max_length=255, verbose_name=u"Titre du sujet")
    url = models.URLField(verbose_name=u"Loomio", blank=True, null=True)

    def __str__(self):
        return self.title

    class Meta:
        serializer_fields = ['@id','title', 'url']


class Theme(Model):
    """Type of ActionGroup"""
    name = models.CharField(max_length=255, verbose_name=u"Nom")
    photo = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Photo")

    def __str__(self):
        return self.name


class OrgType(Model):
    """Type of organisation. E.g. NGO, Citizen Movement.."""
    name = models.CharField(max_length=255, verbose_name=u"Type d'organisation")

    def __str__(self):
        return self.name
  

class representativeContactPlateform(Model):
    name = models.CharField(max_length=255, verbose_name=u"Plateforme d'interpellation")

    def __str__(self):
        return self.name


class actionMapSignUp(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    road = models.CharField(max_length=255,blank=True,null=True,verbose_name=u"Rue du rendez vous")
    city = models.CharField(max_length=255,verbose_name=u"Ville du rendez vous")
    contactName = models.CharField(max_length=255,verbose_name=u"Contact Name")
    link = models.CharField(max_length=255,blank=True, null=True,verbose_name=u"Lien")
    contactEmail = models.CharField(max_length=255,verbose_name=u"Contact Email")
    dateDePassage = models.CharField(max_length=255,default='',verbose_name=u"Date")
    time = models.TimeField(blank=True, null=True,verbose_name=u"Heure du rdv")
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Latitude")
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Longitude")

    class Meta:
        serializer_fields =['@id','created_at','link','dateDePassage','time','road','city','contactEmail','contactName',
        'lat','lng']
        nested_fields=['actionSignUps']
        anonymous_perms=['view','add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['created_at']

    def __str__(self):
        return self.city + " " + self.contactName


class UrlShortner(Model):
    actionName = models.CharField(max_length=200)
    url = models.CharField(max_length=200)
    shortLink = models.CharField(max_length=200)
    class Meta:
        nested_fields = ['actionShortLink']

    def __str__(self):
        print(f"Short Url for: {self.url} is {self.shortLink}")
        if self.actionName :
            return self.actionName 
        else:
            return self.url


        
class ActionGroup(Model):
    """Connected users can create actions that are visible to all users in the group"""
    created_at = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=255, verbose_name=u"Nom de l'action")
    status = models.CharField(max_length=32, default='active', choices=(('active', 'Active'),('pending', 'Pending'), ('inappropriate', 'Inappropriate'), ('finished', 'Finished Action')), verbose_name="action status")
    instanceName = models.CharField(max_length=255, verbose_name=u"Nom de l'instance",default=settings.INSTANCE_NAME)
    instanceHostName = models.CharField(max_length=255, verbose_name=u"Domaine de l'instance",default=settings.INSTANCE_DOMAINE_NAME)
    objective = models.TextField(verbose_name=u"Objectif")
    description = models.TextField(blank=True, null=True, verbose_name=u"Contexte / Description")
    
    actiondate = models.DateField(blank=True, null=True, verbose_name=u"Date de l'action")
    endactiondate = models.DateField(blank=True, null=True, verbose_name=u"Date de fin de l'action")
    image = models.ImageField(verbose_name=u"Image Tablette et Ordinateur", null=True, blank=True)
    mobileImage = models.ImageField(verbose_name=u"Image Mobile", null=True, blank=True)
    shortLink = models.ForeignKey(UrlShortner, verbose_name=u"Short Link",related_name='actionShortLink', null=True,blank=True,
                                          on_delete=models.SET_NULL)
    
    titleOrganisationLogoSection = models.CharField(max_length=255,blank=True, null=True, verbose_name=u"Titre au dessus de la section des logos")
    organisation = models.CharField(max_length=255,blank=True, null=True, verbose_name=u"Organisation associé à l'action")
    emailOrganisation = models.URLField(max_length=9999999, blank=True, null=True,verbose_name=u"Email de contact de l'organisation")
    linkOrganisation = models.URLField(max_length=9999999, blank=True, null=True,verbose_name=u"Lien 'en savoir plus sur le collectif'")
    # more info on action
    actioninfolink = models.URLField(max_length=9999999, blank=True, null=True,verbose_name=u"Lien 'En savoir plus'")
    
    #5 elements below not in admin fieldset
    members = models.ManyToManyField(User, related_name='actiongroups', blank=True)
    actiongroups = models.ManyToManyField("self", blank=True)
    author = models.ForeignKey(User, related_name='created_groups', blank=True, null=True, on_delete=models.SET_NULL)
    conversation = models.ManyToManyField(Conversation, blank=True)
    slug = models.SlugField(blank=True, null=True, unique=True)
    
    
    address = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse")
    city = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Ville")
    postcode = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Code postal")
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Latitude")
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Longitude")
    tags = models.ManyToManyField(Tag,verbose_name=u"Action tags",blank=True,null=True)


    #link to country
    country = models.ForeignKey(Countrie, related_name='action_country', blank=True, null=True, on_delete=models.SET_NULL)
    
    #count emails
    representativeEmailSentCounter = models.IntegerField(default="0",verbose_name=u"Nombre d'email envoyé",blank=True,null=True)
    
    #Representative Targeting choices
    typeOfTargetting = models.CharField(max_length=32,blank=True, default='', choices=(('postalCode', 'Par code postal'),('byGeography', 'Par département'), ('specificRepresentatives', 'Je choisi certaines personnes'),('none', "Pas d'interpellation")), verbose_name="Type de ciblage")
    targetedOfficials = models.ManyToManyField(representativeFunction,blank=True, related_name="targetedRepresentatives", verbose_name=u"Type d'élues ciblés. Si vous ciblez des décisionnaires précis, laissez ce champs vide")
    contactPlateforms = models.ManyToManyField(representativeContactPlateform,blank=True, related_name="contactPlateforms", verbose_name=u"Plateforme d'interpellations")

    pressreleasetitle = models.CharField(max_length=255,blank=True, null=True, verbose_name=u"Communiqué de Presse - Titre")
    pressreleasecontent = models.FileField(blank=True, null=True, verbose_name=u"Communiqué de Presse - PDF")

    # elected officials email
    electedofficialEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Sujet de l'email à  envoyer aux députées")
    electedofficialEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Texte de l'email à envoyer aux députées")
    # senator email
    senatorEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Sujet de l'email à envoyer aux sénateurices")
    senatorEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Texte de l'email à envoyer aux sénateurices")
    # candidate email
    candidateEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Sujet de l'email à envoyer aux candidat.e.s")
    candidateEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Texte de l'email à envoyer aux candidat.e.s")    

    #specific targeted people
    
    specificTargetedRepresentatives = models.ManyToManyField(Representative,verbose_name=u"Cible précisement certains élu-e-s",blank=True,null=True)
    # -> title of section here to allow dynamic
    # mused be informed to display section
    targetedRepresentativesTitle = models.CharField(max_length=255, verbose_name=u"Cible précise - Titre de la section",blank=True, null=True,)
    targetedEmailSubject = models.CharField(max_length=255,blank=True, null=True,
                                                   verbose_name=u"Cible précise - Sujet de l'email")
    targetedEmailText = models.TextField(blank=True, null=True,
                                                verbose_name=u"Cible précise - Texte de l'email")    

    
    #interpellation députée on twitter
    TweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Main Tweet")
    Tweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Tweet 2")
    Tweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Tweet 3")
    Tweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Députée - Tweet 4")
    
    #interpellation senators on twitter
    senatorTweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Main Tweet")
    senatorTweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Tweet 2")
    senatorTweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Tweet 3")
    senatorTweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"senator - Tweet 4")
    
    #interpellation candidates on twitter
    candidateTweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Main Tweet")
    candidateTweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Tweet 2")
    candidateTweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Tweet 3")
    candidateTweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"candidate - Tweet 4")
    
    #interpellation targeted on twitter
    targetedTweetMain = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Main Tweet")
    targetedTweet2 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Tweet 2")
    targetedTweet3 = models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Tweet 3")
    targetedTweet4= models.CharField(blank=True, null=True,max_length=350, verbose_name=u"Cible précise - Tweet 4")
    
    

    #interpellation par téléphone
    representativeCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Députée - Objectif de l'appel")
    representativeCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Députée - Argumentaire pour l'appel")

    senatorCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Sénateurice - Objectif de l'appel")
    senatorCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Sénateurice - Argumentaire pour l'appel")

    candidateCallObjective = models.CharField(blank=True, null=True, max_length=280, verbose_name=u"Candidat - Objectif de l'appel")
    candidateCallArgumentation = models.TextField(blank=True, null=True, verbose_name=u"Candidat - Argumentaire pour l'appel")

    
    #call video
    videotitle = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Titre de la vidéo")
    videourl = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Vidéo associé à l'action")

    #Signups 
    mapSignUpSectionTitle = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Titre de la section Carte")
    mapSignUp = models.ManyToManyField(actionMapSignUp, blank=True, related_name="actionSignUps")
    
    #Action Step title (each step in nested fields)
    actionStepTitle = models.TextField(verbose_name=u"Titre de la section 'étape' de l'action",blank=True,null=True)

    
    class Meta:
        auto_author = 'author'
        serializer_fields = ['@id','shortLink', 'name', 'author', 'pressreleasetitle', 'pressreleasecontent','status',
                             'objective', 'actiondate','organisation','emailOrganisation','linkOrganisation','actioninfolink', 
                             'endactiondate', 'image','mobileImage', 'description','country',
                             'electedofficialEmailSubject', 'electedofficialEmailText','targetedEmailSubject','targetedEmailText',
                             'members','instanceName','instanceHostName',
                             'representativeCallObjective','representativeCallArgumentation','senatorCallObjective','senatorCallArgumentation',
                             'actiongroups', 'conversation', 'address', 'city', 'postcode', 'lat', 'lng',
                             'events', 'org_actions','videotitle',
                             'videourl','TweetMain','Tweet2','Tweet3','Tweet4',
                             'senatorEmailSubject','senatorEmailText',
                             'senatorTweetMain','senatorTweet2','senatorTweet3','senatorTweet4','targetedTweetMain','targetedTweet2','targetedTweet3','targetedTweet4',
                             'candidateEmailSubject','candidateEmailText',
                             'candidateTweetMain','candidateTweet2','candidateTweet3','candidateTweet4',
                             'candidateCallObjective','candidateCallArgumentation',
                             'related_text','typeOfTargetting','targetedOfficials','contactPlateforms',
                             'mapSignUpSectionTitle','mapSignUp','actionStep','representativeEmailSentCounter',
                             'specificTargetedRepresentatives','targetedRepresentativesTitle',
                             'tags',"supportOrganisationLogo","titleOrganisationLogoSection"]
        nested_fields = ["author", "actiongroups", "conversation", "members", "events",
                         'org_actions','related_text','actionEmailSentRelated',
                         "actionStep","supportOrganisationLogo"
                         ]
        anonymous_perms = ['view','add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        owner_field = 'author'
        ordering = ['-created_at']
        view_set = I18nLDPViewSet
        rdf_type = 'risefor:actiongroup'
        container_path = 'actiongroups/'
        # lookup_field = 'id'
    def __str__(self):
        return self.name
###
# Short link creater
###
@receiver(post_save, sender=ActionGroup)
def save_createShortLink_actiongroup(sender, instance, created, **kwargs):
    import random
    try: 
        existingUrls = UrlShortner.objects.get(url=instance.urlid)    
    except UrlShortner.DoesNotExist:
        actionName = instance.name.replace(" ","-")
        actionName = actionName.replace(",","")
        actionName = actionName.replace("'","")
        actionName = actionName.replace("?","")
        actionName = actionName.replace("|","")
        actionName = actionName.replace("(","")
        actionName = actionName.replace(")","")
        actionName = actionName.lower()
        actionName = cleanAccentuation(actionName)
        # print("new action name",actionName)
        actionSlug = settings.INSTANCE_DOMAINE_NAME + "/s/" + actionName + "-" + str(random.randrange(1,10000))
        
        shortUrl = UrlShortner(
            actionName="Lien pour l'action : "+instance.name,
            url=instance.urlid,shortLink = actionSlug)
        shortUrl.save()

#short Link created, get it & update group value
@receiver(post_save, sender=ActionGroup)
def save_updateGroupShortLink_actiongroup(sender, instance, created, **kwargs):
    # print('shortlink field',instance.shortLink)
    if instance.shortLink == None:
        existingShortLinks = UrlShortner.objects.all()
        actionGroupLink = existingShortLinks.get(url=instance.urlid)
        instance.shortLink = actionGroupLink
        instance.save(update_fields=['shortLink'])

#Mobile image handling - If no image provided, take the desktop image and save it as mobile version
@receiver(post_save, sender=ActionGroup)
def save_emptyMobileImage_actiongroup(sender, instance, created, **kwargs):
    if (instance.mobileImage == None) or (instance.mobileImage == '') :
        #if no desktop image, just save 
        if (instance.image == None) or (instance.image == '') :
            instance.save
        #else, get the desktop image & save it as mobile
        else:
            desktopImage = instance.image
            print("empty mobile image",desktopImage) 
            instance.mobileImage = desktopImage
            instance.save(update_fields=['mobileImage'])
            print("mobile image updated",instance.mobileImage) 
        
        # existingShortLinks = UrlShortner.objects.all()
        # actionGroupLink = existingShortLinks.get(url=instance.urlid)
        # instance.shortLink = actionGroupLink
        # instance.save(update_fields=['shortLink'])


class OrganisationLogoForAction(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    orgName = models.CharField(max_length=255, verbose_name=u"Nom de l'organisation") 
    image = models.ImageField(verbose_name=u"Logo", null=False, blank=False)
    linkedAction = models.ForeignKey(ActionGroup, related_name='supportOrganisationLogo',null=True, blank=False, on_delete=models.SET_NULL)

    class Meta:
        serializer_fields =['@id','created_at','orgName','image','linkedAction']
        # nested_fields=['']
        anonymous_perms=['view']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        ordering = ['created_at']

    def __str__(self):
        return self.orgName 


class ActionStep(Model):
    order = models.IntegerField(verbose_name=u"Ordre (1,2,3)")
    name = models.CharField(max_length=50, blank=True, null=True, verbose_name=u"Titre de l'étape")
    description =  models.TextField(blank=True, null=True, verbose_name=u"Description rapide")
    actiongroup = models.ForeignKey(ActionGroup,blank=True, null=True, related_name="actionStep", verbose_name=u"Action associé", on_delete=models.SET_NULL)
    active = models.BooleanField(default=False,verbose_name=u"Etape en cours")

    def __str__(self):
        return self.name

class ElectedOfficialContact(Model):
    author = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)
    created_at = models.DateTimeField(auto_now_add=True)
    subject = models.CharField(max_length=255,verbose_name=u"Sujet de l'email envoyé aux élu.e.s")
    content = models.TextField(verbose_name=u"Contenu de l'email envoyé aux élu.e.s")
    userName = models.TextField(verbose_name=u"Name of citizen")
    userEmail = models.CharField(max_length=255, verbose_name=u"User email")
    representativeEmails = models.CharField(max_length=500, verbose_name=u"Adresses ciblés")
    linkedAction = models.ManyToManyField(ActionGroup,blank=True,related_name="actionEmailSentRelated")
    class Meta:
        auto_author = 'author'
        serializer_fields =['@id','author','subject','content','userName','userEmail','representativeEmails','linkedAction']
        nested_fields=['linkedAction']
        anonymous_perms=['view','add']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        owner_field = 'author'
        ordering = ['created_at']

        view_set = I18nLDPViewSet
        rdf_type = 'risefor:contactRepresentatives'
        container_path = 'electedofficialcontacts/'
    
    def __str__(self):
        return self.userName



class Organisation(Model):
    """Organisations can add a page describing their organisation and actions"""
    created_at = models.DateTimeField(auto_now_add=True)
    organisationName = models.CharField(max_length=255, verbose_name=u"Nom de l'association")
    tagline = models.CharField(blank=True, null=True, max_length=600, verbose_name=u"Tagline")
    author = models.ForeignKey(User, related_name='created_ngo', blank=True, null=True, on_delete=models.SET_NULL)
    members = models.ManyToManyField(User, related_name='ngomembers', blank=True)
    description = models.TextField(blank=True, null=True, verbose_name=u"Description")
    site = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Site du collectif")
    telephone = models.CharField(max_length=50, blank=True, null=True, verbose_name=u"Numéro de téléphone")
    contact_address = models.EmailField(max_length=400, blank=True, null=True, verbose_name=u"NGO Contact email")
    org_actions = models.ManyToManyField(ActionGroup, blank=True, related_name='org_actions')
    logo = models.CharField(default="/static/img/action_group_blank.png", blank=True, null=True,
                            verbose_name="Logo du collectif", max_length=1000)
    slug = models.SlugField(blank=True, null=True, unique=True)
    address = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Adresse")
    city = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Ville")
    postcode = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Code postal")
    lat = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Latitude")
    lng = models.DecimalField(max_digits=15, decimal_places=12, blank=True, null=True, verbose_name=u"Longitude")
    organisation_type = models.ForeignKey(OrgType, verbose_name=u"Type d'organisation", null=True,
                                          on_delete=models.SET_NULL)
    facebook = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Facebook Account")
    twitter = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Twitter Account")
    instagram = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"Instagram Account")
    linkedin = models.URLField(max_length=9999999, blank=True, null=True, verbose_name=u"LinkedIn Account")

    class Meta:
        auto_author = 'author'
        serializer_fields = ['@id', 'organisationName', 'facebook', 'twitter', 'instagram', 'linkedin', 'organisation_type',
                             'tagline', 'members', 'contact_address', 'telephone', 'org_actions', 'logo', 'author',
                             'site', 'description', 'address', 'city', 'postcode', 'lat', 'lng']
        nested_fields = ["author", 'members', 'org_actions']
        anonymous_perms = ['view']
        authenticated_perms = ['inherit', 'add']
        owner_perms = ['inherit', 'change', 'control', 'delete']
        owner_field = 'author'
        ordering = ['-created_at']

    def __str__(self):
        return self.organisationName


class Event(Model):
    """An event, as associated with an Action"""
    eventName = models.CharField(max_length=255, verbose_name=u"Nom de l'évenement")
    image = models.URLField(verbose_name=u"Image", null=True, blank=True)
    date = models.DateField(blank=True, null=True, verbose_name=u"Date de l'évenement")
    location = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Lieu")
    theme = models.ForeignKey(Theme, verbose_name=u"Type d'événement", null=True, on_delete=models.SET_NULL)
    organization = models.CharField(max_length=255, blank=True, null=True, verbose_name=u"Organisation")
    actiongroup = models.ForeignKey(ActionGroup, related_name="events", blank=True, null=True, on_delete=models.SET_NULL)
    eventDescription = models.TextField(blank=True, null=True, verbose_name=u"Description")

    class Meta:
        serializer_fields = ['@id','image', 'eventName', 'eventDescription', 'date', 'location', 'theme', 'actiongroup', 'organization']
        ordering=['-date']

    def __str__(self):
        return self.eventName

# HelpInstall - one kind of contact form (submitting a request for help installing the tool)


class HelpOrgType(Model):
    """Type of organisation wanting to install the tool"""
    name = models.CharField(max_length=255, verbose_name=u"Type de collectif")

    def __str__(self):
        return self.name


class HelpLevel(Model):
    """A category representing the amount of help needed"""
    name = models.CharField(max_length=255, verbose_name=u"Niveau d'aide nécessaire")

    def __str__(self):
        return self.name


class HelpInstall(Model):
    """Contact form to ask for help in order to deploy Risefor"""
    created_at = models.DateTimeField(auto_now_add=True)
    orgname = models.CharField(max_length=255, verbose_name=u"Nom de l'organisation")
    orgType = models.ForeignKey(HelpOrgType, verbose_name=u"Type de collectif", null=True, on_delete=models.SET_NULL)
    helpLevel = models.ForeignKey(HelpLevel, verbose_name=u"Level of technical help needed", null=True, on_delete=models.SET_NULL)
    usage_tool = models.TextField(verbose_name=u"Cadre d'utilisation")
    extra_msg = models.TextField(verbose_name=u"Message supplémentaire")
    contactEmail = models.EmailField(max_length=400, verbose_name=u"Adresse email du contact")

    class Meta:
        serializer_fields = ['@id', 'created_at', 'extra_msg', 'usage_tool', 'orgname', 'orgType', 'helpLevel',
                             'contactEmail']
        anonymous_perms = ['view', 'add']
        authenticated_perms = ['inherit', 'add']

    def __str__(self):
        return self.contactEmail


# ContactTeam is used for other kinds of contact


class ContactReason(Model):
    """Reasons for making contact with the team"""
    name = models.CharField(max_length=255, verbose_name=u"Raison du contact")

    def __str__(self):
        return self.name


class ContactTeam(Model):
    """A submitted contact form"""
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=u"Date d'envoie")
    subject = models.CharField(max_length=255, verbose_name=u"Sujet de l'email")
    contact_reason = models.ForeignKey(ContactReason, verbose_name=u"Raison du contact", null=True,
                                       on_delete=models.SET_NULL)
    content = models.TextField(verbose_name=u"Contenu de l'email")
    contact_email = models.EmailField(max_length=400, verbose_name=u"Adresse email du contact")

    class Meta:
        serializer_fields = ['@id', 'created_at','subject','contact_reason', 'content', 'contact_email']
        # nested_fields=['contact_reason']
        anonymous_perms = ['view', 'add']
        authenticated_perms = ['inherit', 'add']

    def __str__(self):
        return self.subject


# TODO: this model is a workaround to https://git.happy-dev.fr/startinblox/solid-spec/issues/14
# it allows us to have permissions specifically for the email field
class TextSupporters_Email(Model):
    """Email of supports that signed the txt in an action group"""
    email = models.EmailField(max_length=400, blank=True, null=True, verbose_name=u"Supporter email")

    class Meta:
        serializer_fields = ['email']
        anonymous_perms = ['add', 'view']

    def __str__(self):
        return self.email




class TextSupporter(Model):
    """Allow citizens to sign a text that is in an action group"""
    created_at = models.DateTimeField(auto_now_add=True)
    supporterFirstname = models.CharField(max_length=250, verbose_name=u"Prénom")
    supporterLastname = models.CharField(max_length=250, verbose_name=u"Nom")
    supporterCollectif = models.CharField(max_length=250,blank=True,null=True, verbose_name=u"Organisation")
    supportedGroup = models.ForeignKey(ActionGroup, related_name="supportedGroup",blank=True,null=True, on_delete=models.SET_NULL)
    supportedEmail = models.CharField(max_length=250, verbose_name=u"Email",blank=True,null=True)
    #supportedEmail = models.ForeignKey(TextSupporters_Email, related_name="supported_email", on_delete=models.CASCADE,blank=True, null=True)
    class Meta:
        serializer_fields = ['@id','supportedEmail','supportedGroup','supporterFirstname','supporterLastname','supporterCollectif']
        nested_fields=['supportedGroup']
        anonymous_perms = ['add','view']
        authenticated_perms = ['inherit', 'add']
        ordering=['-created_at']

    def __str__(self):
        return self.supporterFirstname + " " + self.supporterLastname



class Newsletter(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    firstname = models.CharField(max_length=250, verbose_name=u"First name")
    lastname = models.CharField(max_length=250, verbose_name=u"Last name")
    userEmail = models.EmailField(max_length=200, verbose_name=u"Email du citoyen.ne")
    subscriptionType = models.TextField(verbose_name=u"Type de souscription",blank=True,null=True)

    class Meta:
        serializer_fields = ['@id','subscriptionType','firstname','userEmail','lastname']
        anonymous_perms = ['add','view']
        authenticated_perms = ['inherit', 'add','view']
        ordering = ['-created_at']

    def __str__(self):
        return self.firstname + " " + self.lastname


class InformNewsletterOption(Model):
    name = models.CharField(max_length=255, verbose_name=u"Subscription to the newsletter")

    def __str__(self):
        return self.name
                                                                                                       



class HomePageElectedOfficals(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    firstname = models.CharField(max_length=250, verbose_name=u"First name")
    lastname = models.CharField(max_length=250, verbose_name=u"Last name")
    userEmail = models.EmailField(max_length=200, verbose_name=u"Email du citoyen.ne")
    emailSubject = models.CharField(max_length=255, verbose_name=u"Sujet")
    emailContent =   models.TextField(verbose_name=u"Contenu de l'email")
    electedEmails = models.TextField(verbose_name=u"Emails des élues")
    newsLetterSignUp = models.ForeignKey(InformNewsletterOption, null=True, on_delete=models.SET_NULL)

    class Meta:
        serializer_fields = ['@id','newsLetterSignUp','firstname','electedEmails','emailContent','emailSubject','userEmail','lastname']
        anonymous_perms = ['add','view']
        authenticated_perms = ['inherit','add']
        ordering = ['-created_at']

    def __str__(self):
        return f"{self.firstname} {self.lastname}"

class ContactEO(Model):
    """Enable citizens to send a personalized email on the base of an offered text within an action group"""
    created_at = models.DateTimeField(auto_now_add=True)
    # user info
    citizen_firstname = models.CharField(max_length=255, verbose_name=u"Prénom")
    citizen_lastname = models.CharField(max_length=255, verbose_name=u"Nom")
    # linked data
    related_action = models.ForeignKey(ActionGroup, related_name='related_text', blank=True, null=True, on_delete=models.SET_NULL)
    targeted_official = models.ManyToManyField(Representative, related_name='linked_official')
    # email content
    email_subject = models.CharField(max_length=255, verbose_name=u"Sujet")
    email_content = models.TextField(verbose_name=u"Contenu de l'email")
    email_citizen = models.EmailField(max_length=200, verbose_name=u"Email du citoyen.ne")
    # for security email is it's own model with own permissions
    # citizen_email = models.ManyToManyField(ContactEO_Email, related_name='citizen_email')
    # Can't fetch Citizens manytomany on email send// https://git.happy-dev.fr/startinblox/applications/risefor-lobbying/issues/389
    elected_email = models.TextField(max_length=400, verbose_name=u"Contacted elected official ")

    class Meta:
        serializer_fields = ['@id', 'email_subject', 'email_content', 'targeted_official', 'related_action',
                             'citizen_lastname', 'email_citizen', 'elected_email', 'citizen_firstname']
        nested_fields = []
        anonymous_perms = ['view', 'add']
        authenticated_perms = ['inherit', 'add', 'view']
        owner_perms = ['inherit', 'change', 'control']

    def __str__(self):
        return self.citizen_firstname + " " + self.citizen_lastname


#Email des Signataire de l'appel en Home Page <-- Modèle dédié pour p    rotéger les données
class CitizenSupportEmail(Model):
  email = models.EmailField(max_length=250, blank=True, null=True, verbose_name=u"Supporter email")
  class Meta:
    serializer_fields = ['email']
    nested_fields = ['signataireEmail']
    authenticated_perms = ['inherit', 'add']
    anonymous_perms = ['add']

    """Anoted below because creates error when trying to access  
    citizensupport model in admin
    
    def __str__(self):
    return self.email
      """


"""Citizens available to come help the inhabitants when needed"""
class CitizenSupport(Model):
    created_at = models.DateTimeField(auto_now_add=True)
    firstname = models.CharField(max_length=255, verbose_name=u"Prénom") 
    lastname = models.CharField(max_length=255, verbose_name=u"Nom")     
    #slug = models.SlugField(blank=True, null=True, unique=True)
    signataireEmail = models.ForeignKey(CitizenSupportEmail, related_name="signataireEmail", blank=True, null=True,
                                        on_delete=models.SET_NULL)
    #contact info taken via profile or allow anyone to signup?

    class Meta:
        serializer_fields = ['@id','signataireEmail','firstname','lastname']
        anonymous_perms = ['view',"add"]
        authenticated_perms = ['inherit', 'add']
        ordering=['-created_at']

        def __str__(self):
            return self.name




@receiver(post_save, sender=ActionGroup)
def save_autojoin_actiongroup(sender, instance, created, **kwargs):
    if created and instance.author is not None:
        instance.members.add(instance.author)


@receiver(post_save, sender=Organisation)
def save_autojoin_organisation(sender, instance, created, **kwargs):
    if created and instance.author is not None:
        instance.members.add(instance.author)


@receiver(pre_save, sender=ActionGroup)
def actiongroup_generate_slug(sender, instance, **kwargs):
    instance.slug = default_slug = slugify(instance.name)[:40]
    for index in itertools.count(1):
        if not ActionGroup.objects.filter(slug=instance.slug).exists():
            break
        instance.slug = "{}-{}".format(default_slug, index)


@receiver(pre_save, sender=Organisation)
def organisation_generate_slug(sender, instance, **kwargs):
    instance.slug = default_slug = slugify(instance.organisationName)[:40]
    for index in itertools.count(1):
        if not Organisation.objects.filter(slug=instance.slug).exists():
            break
        instance.slug = "{}-{}".format(default_slug, index)


@receiver(pre_save, sender=Representative)
def representative_generate_slug(sender, instance, **kwargs):
    instance.slug = default_slug = slugify("{} {}".format(instance.firstname, instance.lastname))[:40]
    for index in itertools.count(1):
        if not Representative.objects.filter(slug=instance.slug).exists():
            break
        instance.slug = "{}-{}".format(default_slug, index)


@receiver(post_save, sender=ContactEO)
def send_email_on_contacteo(sender, instance, created, **kwargs):
    if created:
        teamEmail = "community@risefor.org"
        body = {
        'Sujet ': instance.email_subject, 
        'Message  ': instance.email_content,
        'De la part de ': instance.citizen_firstname + " " + instance.citizen_lastname,
        
         }
        content = {"%s: %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        email = EmailMessage(
        instance.email_subject,
        content, 
        from_email=[instance.email_citizen],
        to=[instance.elected_email],
        cc=[instance.email_citizen,teamEmail],
        reply_to=[instance.email_citizen,teamEmail],
        )
        email.send()


@receiver(post_save, sender=HomePageElectedOfficals)
def send_email_on_HomePageElectedOfficals(sender, instance, created, **kwargs):
    if created:
        #teamEmail = "test@test.com"
        key = [' ','De la part ']
        body = {
            key[0] : instance.emailContent,
            key[1] : instance.firstname + " " + instance.lastname,
        }
        content = {"%s : %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        content =content.strip(key[0] + " :")
        email = EmailMessage(
            instance.emailSubject + " " + instance.firstname + " "+ instance.lastname,
            content,
            instance.userEmail,#here is from value
            to=[instance.electedEmails],
            cc=[instance.userEmail],
            reply_to=[instance.userEmail],
            #from=[instance.userEmail],
        )
        #email.attach_file('./static/img/ATTESTATION-DE-SORTIE-DE-CRISE-DÉROGATOIRE.pdf')
        email.send()


@receiver(post_save, sender=ElectedOfficialContact)
def send_email_on_ElectedOfficialContact(sender, instance, created, **kwargs):
    if created:
        #teamEmail = "test@test.com"
        key = ['','De la part de']
        body = {
            key[0] : instance.content,
            key[1] : instance.userName,
        }
        content = {"%s : %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        content =content.strip(key[0] + " :")
        email = EmailMessage(
            instance.subject + " " + instance.userName,
            content,
            instance.userEmail,#here is from value
            to=[instance.representativeEmails],
            cc=[instance.userEmail],
            reply_to=[instance.userEmail],
        )
        #email.attach_file('./static/img/ATTESTATION-DE-SORTIE-DE-CRISE-DÉROGATOIRE.pdf')
        email.send()



#update count of emails sent to in all actions
@receiver(post_save, sender=ElectedOfficialContact)
def save_updateRepresentativeEmailCounter_actiongroup(sender, instance, created, **kwargs):
    if created:
        actionGroups = ActionGroup.objects.all()
        for actionGroup in actionGroups:
            actionGroup.representativeEmailSentCounter = actionGroup.actionEmailSentRelated.count()
            if actionGroup.representativeEmailSentCounter != 0:
                actionGroup.representativeEmailSentCounter = actionGroup.representativeEmailSentCounter + 1
            print("groups",actionGroup.name,actionGroup.representativeEmailSentCounter)
            actionGroup.save()

@receiver(post_save, sender=ContactTeam)
def send_email_on_contactteam(sender, instance, created, **kwargs):
    if created:
        body = {
            'subject': instance.subject,
            'contact_reason': instance.contact_reason,
            'content': instance.content,
            'contact_email': instance.contact_email,
        }
        content = {"%s: %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        email = EmailMessage(
            "Nouveau Contact " + instance.subject,
            content,
            to=['community@risefor.org']
        )
        email.send()


@receiver(post_save, sender=HelpInstall)
def send_email_on_contact(sender, instance, created, **kwargs):
    if created:
        body = {
            'Nom de organisation': instance.orgname,
            'Type organisation': instance.orgType,
            'Niveau aide nécessaire': instance.helpLevel,
            'Utilisation de outil': instance.usage_tool,
            'Commentaire': instance.extra_msg,
            'Email de contact': instance.contactEmail,

        }
        content = {"%s: %s" % (key, value) for (key, value) in body.items()}
        # turn those strings into 1 block of text separated by newlines
        content = "\n".join(content)
        email = EmailMessage(
            "Demande d'aide par " + instance.orgname,
            content,
            to=['community@risefor.org']
        )
        email.send()

