#from typing import OrderedDict
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.serializers import ValidationError
from rest_framework.response import Response

from djangoldp.views import LDPViewSet
from risefor_lobbying.models import ElectedOfficialContact,ActionGroup, Representative, Organisation, HomePageElectedOfficals
from datetime import date,datetime
from django.conf import settings

class RepresentativeViewSet(LDPViewSet):
     model=Representative

     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).exclude(function__icontains="attache")

class RepresentativeSearchViewSet(LDPViewSet):
    model=Representative
    fields = ['@id','fullname','photo','politicalParty','function','allPostalCodes','directEmails','twitterAccount']

class ActionViewSet(LDPViewSet):
    model=ActionGroup
    def get_queryset(self, *args, **kwargs):
        querySet = super().get_queryset(*args, **kwargs)
        localActiveActions = querySet.filter(status__contains='active')
        print("local actions are",localActiveActions)
        return localActiveActions

class passedActionViewSet(LDPViewSet):
     model=ActionGroup

     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).filter(status__contains='finished')


class OrganisationViewSet(LDPViewSet):
     model=Organisation

     def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).exclude(actiondate__lt=date.today())


class ConvergenceViewSet(LDPViewSet):
    '''
    ViewSet using Actions between two parameterised dates. If only one date is passed then
    only one is applied, i.e. if you pass start_date without end_date it will return all
    Actions after the start_date
    '''
    model=ActionGroup

    def get_queryset(self, *args, **kwargs):
        start_date = self.request.query_params.get('start_date')
        end_date = self.request.query_params.get('end_date')

        queryset = super().get_queryset(*args, **kwargs)

        if start_date is not None:
            try:
                start_date = datetime.strptime(start_date, '%Y-%m-%d')
            except ValueError:
                raise ValidationError('start_date was in an invalid format. Should be YYYY-MM-DD')
            queryset = queryset.filter(actiondate__gt=start_date)

        if end_date is not None:
            try:
                end_date = datetime.strptime(end_date, '%Y-%m-%d')
            except ValueError:
                raise ValidationError('end_date was in an invalid format. Should be YYYY-MM-DD')
            queryset = queryset.filter(actiondate__lt=end_date)

        return queryset

#view created to improve loading time of total messages sent on HP
class TotalSubmission(LDPViewSet):
    model = HomePageElectedOfficals
    
    def get_queryset(self, *args, **kwargs):
        allEmailsSent = HomePageElectedOfficals.objects.all()
        return allEmailsSent

    def list(self, request, *args, **kwargs):
        return Response({'total': self.get_queryset().count()})


class representativesEmailContact(LDPViewSet):
    model = ActionGroup

    def list(self, request,model=model, *args, **kwargs):
        allResults = {}
        count = 0
        for action in model.objects.all():
        #    print("the action is here",action)
            sentEmails = action.actionEmailSentRelated.all().count()
            actionName = action.name
            #actionCounter = [actionName,sentEmails]
            actionCounter = str(sentEmails)
            #print("emails sent",actionCounter,count)
            allResults[actionName] = actionCounter
            count += 1
        #print("actioncount",json.dumps(allResults))
        #print("type is",type(allResults))
        return Response(allResults)


def shortLinkRedirect(request, slugs):
    from risefor_lobbying.models import UrlShortner
    from django.shortcuts import redirect
    print("Short like redirect - requested slug",slugs,settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs)
    
    print("shorlink result",UrlShortner.objects.get(shortLink=settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs) )
    #search for matching url in shortners

    try:
        data = UrlShortner.objects.get(shortLink=settings.INSTANCE_DOMAINE_NAME+'/s/'+slugs)
        frontUrl = data.url.replace(":","%3A")
        print("data url",data)
        #take URL and format it to front end Urls
        finalUrl =  "/actions/action-group-detail/@"+frontUrl +"@"
        print("final url",finalUrl)
        #redirect
        return redirect(finalUrl)
    #if no match, redirect to actions listing
    except (UrlShortner.DoesNotExist) or None:
        print("Short link redirect - except",UrlShortner)
        return redirect("/actions")
